# coding: utf-8

"""jycm codes. .
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from .__version__ import __version__  # NOQA

__author__ = 'eggachecat <sunao_0626@hotmail.com>'
